﻿using System.Collections.Generic;

namespace ProgramTree
{
    public enum AssignType { Assign, AssignPlus, AssignMinus, AssignMult, AssignDivide };

    public class Node // базовый класс для всех узлов    
    {
    }

    public class ExprNode : Node // базовый класс для всех выражений
    {
    }

    public class IdNode : ExprNode
    {
        public string Name { get; set; }
        public IdNode(string name) { Name = name; }
    }

    public class IntNumNode : ExprNode
    {
        public int Num { get; set; }
        public IntNumNode(int num) { Num = num; }
    }

    public class StatementNode : Node // базовый класс для всех операторов
    {
    }

    public class AssignNode : StatementNode
    {
        public IdNode Id { get; set; }
        public ExprNode Expr { get; set; }
        public AssignType AssOp { get; set; }
        public AssignNode(IdNode id, ExprNode expr, AssignType assop = AssignType.Assign)
        {
            Id = id;
            Expr = expr;
            AssOp = assop;
        }
    }

    public class CycleNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public CycleNode(ExprNode expr, StatementNode stat)
        {
            Expr = expr;
            Stat = stat;
        }
    }

    public class BlockNode : StatementNode
    {
        public List<StatementNode> StList = new List<StatementNode>();
        public BlockNode(StatementNode stat)
        {
            Add(stat);
        }
        public void Add(StatementNode stat)
        {
            StList.Add(stat);
        }
    }
    public class WhileNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public WhileNode(ExprNode expr, StatementNode stat)
        {
            Expr = expr;
            Stat = stat;
        }
    }

    public class RepeatNode : StatementNode
    {
        public List<StatementNode> StList = new List<StatementNode>();
        public ExprNode Expr { get; set; }

        public RepeatNode(StatementNode stat, ExprNode expr)
        {
            Add(stat);
            Expr = expr;
        }
        public void Add(StatementNode stat)
        {
            StList.Add(stat);
        }

    }

    public class ForNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode Stat { get; set; }
        public AssignNode Assign { get; set; }
        public ForNode(AssignNode assign, ExprNode expr, StatementNode stat)
        {
            Expr = expr;
            Stat = stat;
            Assign = assign;
        }

    }

    public class IfNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public StatementNode IfStat { get; set; }

        public StatementNode ElseStat { get; set; }
        public IfNode(ExprNode expr, StatementNode ifStat, StatementNode elseStat = null)
        {
            Expr = expr;
            IfStat = ifStat;
            ElseStat = elseStat;
        }
    }

    public class WriteNode : StatementNode
    {
        public ExprNode Expr { get; set; }
        public WriteNode(ExprNode expr)
        {
            Expr = expr;
        }
    }

    public class VarDefNode : StatementNode
    {
        public List<IdNode> IDList = new List<IdNode>();
        public ExprNode Expr { get; set; }
        public VarDefNode(IdNode id)
        {
            Add(id);
        }
        public void Add(IdNode id)
        {
            IDList.Add(id);
        }
    }

    public class BinaryNode : ExprNode
    {
        public ExprNode RightExpr { get; set; }

        public ExprNode LeftExpr { get; set; }

        public char Operation { get; set; }
        public BinaryNode(ExprNode rightExpr, ExprNode leftExpr, char operation)
        {
            RightExpr = rightExpr;
            LeftExpr = leftExpr;
            Operation = operation;
        }
    }
}